summary refs log tree commit diff
path: root/crypto/src/asn1/DerT61String.cs
diff options
context:
space:
mode:
authorPeter Dettman <peter.dettman@bouncycastle.org>2021-11-18 13:46:18 +0700
committerPeter Dettman <peter.dettman@bouncycastle.org>2021-11-18 13:46:18 +0700
commit4fccb8f490eefe7181558b2c3376ab23c33632ee (patch)
treefe8c1877bb5567cb4823823af77e5c8a38423db8 /crypto/src/asn1/DerT61String.cs
parentASN.1: Staged encoding (diff)
downloadBouncyCastle.NET-ed25519-4fccb8f490eefe7181558b2c3376ab23c33632ee.tar.xz
ASN.1: Port of bc-java TYPE instances
- we use Meta.Instance here due to syntax restrictions
- also reworked some ASN.1 string types
Diffstat (limited to 'crypto/src/asn1/DerT61String.cs')
-rw-r--r--crypto/src/asn1/DerT61String.cs116
1 files changed, 70 insertions, 46 deletions
diff --git a/crypto/src/asn1/DerT61String.cs b/crypto/src/asn1/DerT61String.cs
index 64a9bd469..a0e4f1d22 100644
--- a/crypto/src/asn1/DerT61String.cs
+++ b/crypto/src/asn1/DerT61String.cs
@@ -1,4 +1,5 @@
 using System;
+using System.IO;
 
 using Org.BouncyCastle.Utilities;
 
@@ -10,97 +11,120 @@ namespace Org.BouncyCastle.Asn1
     public class DerT61String
         : DerStringBase
     {
-		private readonly string str;
+        internal class Meta : Asn1UniversalType
+        {
+            internal static readonly Asn1UniversalType Instance = new Meta();
+
+            private Meta() : base(typeof(DerT61String), Asn1Tags.T61String) {}
+
+            internal override Asn1Object FromImplicitPrimitive(DerOctetString octetString)
+            {
+                return CreatePrimitive(octetString.GetOctets());
+            }
+        }
 
 		/**
          * return a T61 string from the passed in object.
          *
          * @exception ArgumentException if the object cannot be converted.
          */
-        public static DerT61String GetInstance(
-            object obj)
+        public static DerT61String GetInstance(object obj)
         {
             if (obj == null || obj is DerT61String)
             {
                 return (DerT61String)obj;
             }
+            else if (obj is IAsn1Convertible)
+            {
+                Asn1Object asn1Object = ((IAsn1Convertible)obj).ToAsn1Object();
+                if (asn1Object is DerT61String)
+                    return (DerT61String)asn1Object;
+            }
+            else if (obj is byte[])
+            {
+                try
+                {
+                    return (DerT61String)Meta.Instance.FromByteArray((byte[])obj);
+                }
+                catch (IOException e)
+                {
+                    throw new ArgumentException("failed to construct T61 string from byte[]: " + e.Message);
+                }
+            }
 
             throw new ArgumentException("illegal object in GetInstance: " + Platform.GetTypeName(obj));
         }
 
         /**
-         * return an T61 string from a tagged object.
+         * return a T61 string from a tagged object.
          *
          * @param obj the tagged object holding the object we want
          * @param explicitly true if the object is meant to be explicitly
          *              tagged false otherwise.
          * @exception ArgumentException if the tagged object cannot
          *               be converted.
          */
-        public static DerT61String GetInstance(
-            Asn1TaggedObject	obj,
-            bool				isExplicit)
+        public static DerT61String GetInstance(Asn1TaggedObject taggedObject, bool declaredExplicit)
         {
-			Asn1Object o = obj.GetObject();
+            return (DerT61String)Meta.Instance.GetContextInstance(taggedObject, declaredExplicit);
+        }
 
-			if (isExplicit || o is DerT61String)
-			{
-				return GetInstance(o);
-			}
+        private readonly byte[] m_contents;
 
-			return new DerT61String(Asn1OctetString.GetInstance(o).GetOctets());
+        public DerT61String(string str)
+        {
+			if (str == null)
+				throw new ArgumentNullException("str");
+
+            m_contents = Strings.ToByteArray(str);
         }
 
-        /**
-         * basic constructor - with bytes.
-         */
-        public DerT61String(
-            byte[] str)
-			: this(Strings.FromByteArray(str))
-		{
+        public DerT61String(byte[] contents)
+            : this(contents, true)
+        {
         }
 
-		/**
-         * basic constructor - with string.
-         */
-        public DerT61String(
-            string str)
+        internal DerT61String(byte[] contents, bool clone)
         {
-			if (str == null)
-				throw new ArgumentNullException("str");
+            if (null == contents)
+                throw new ArgumentNullException("contents");
 
-			this.str = str;
+            m_contents = clone ? Arrays.Clone(contents) : contents;
         }
 
-		public override string GetString()
+        public override string GetString()
         {
-            return str;
+            return Strings.FromByteArray(m_contents);
         }
 
         internal override IAsn1Encoding GetEncoding(int encoding)
         {
-            return new PrimitiveEncoding(Asn1Tags.Universal, Asn1Tags.T61String, GetOctets());
+            return new PrimitiveEncoding(Asn1Tags.Universal, Asn1Tags.T61String, m_contents);
         }
 
         internal override IAsn1Encoding GetEncodingImplicit(int encoding, int tagClass, int tagNo)
         {
-            return new PrimitiveEncoding(tagClass, tagNo, GetOctets());
+            return new PrimitiveEncoding(tagClass, tagNo, m_contents);
         }
 
         public byte[] GetOctets()
         {
-			return Strings.ToByteArray(str);
+            return Arrays.Clone(m_contents);
         }
 
-		protected override bool Asn1Equals(
-			Asn1Object asn1Object)
-		{
-			DerT61String other = asn1Object as DerT61String;
+        protected override bool Asn1Equals(Asn1Object asn1Object)
+        {
+            DerT61String that = asn1Object as DerT61String;
+            return null != that
+                && Arrays.AreEqual(this.m_contents, that.m_contents);
+        }
 
-			if (other == null)
-				return false;
+        protected override int Asn1GetHashCode()
+        {
+            return Arrays.GetHashCode(m_contents);
+        }
 
-            return this.str.Equals(other.str);
+        internal static DerT61String CreatePrimitive(byte[] contents)
+        {
+            return new DerT61String(contents, false);
         }
-	}
+    }
 }
@@ -10,97 +11,120 @@ namespace Org.BouncyCastle.Asn1
          *
-         * @param obj the tagged object holding the object we want
-         * @param explicitly true if the object is meant to be explicitly
-         *              tagged false otherwise.
-         * @exception ArgumentException if the tagged object cannot
-         *               be converted.
+         * @param taggedObject the tagged object holding the object we want
+         * @param declaredExplicit true if the object is meant to be explicitly tagged false otherwise.
+         * @exception ArgumentException if the tagged object cannot be converted.
          */